module.exports = function (grunt) {

	var backendModules = ['../../teknisa/cmp'];
	var directories = ['repositories', 'services', 'controllers'];
	var subModules = ['fornecimento', 'parametros', 'solicitacao', 'V1', 'necessidadeCompras', 'cadastros', 'simplificada',
	
		// SubModulo FORNECIMENTO 
		'fornecimento/assets/modules/apuracaoEfetivacao',
		'fornecimento/assets/modules/autorizacaoFornecimento',
		'fornecimento/assets/modules/emissaoAutorizacaoFornecimento',
		'fornecimento/assets/modules/listaPreco',
		'fornecimento/assets/modules/migracaoAF',
		'fornecimento/assets/modules/emissaoAF',
		
		// SubModulo SOLICITACAO
		'solicitacao/assets/modules/relatorios',
		'solicitacao/assets/modules/solicitacaoCompras',
		'solicitacao/assets/modules/consolidacao'
	];
	
	var libs = [
		"./bower_components/zh-reports/dist/zhReports.js",
		"./bower_components/zh-grid-configuration/dist/zh-grid-configuration.js",
		"./bower_components/zh-preferences/dist/preferences.js",
		"./bower_components/zh-filter-preferences/dist/filter-preferences.js",
        "./bower_components/zh-layout-preferences/dist/layout-preferences.js",
        "./bower_components/zh-view-preferences/dist/view-preferences.js",
        "./bower_components/zh-rich-text/dist/zh-rich-text.js",
        "./bower_components/zh-select-autocomplete/dist/zh-select-autocomplete.js",
        "./bower_components/zh-interval-component/dist/zh-interval-component.js",
        "./bower_components/zh-grid-navigation/dist/GridNavigationLibrary.js",
		"./bower_components/zh-cache-bypass/src/CacheBypass.js",
		"./bower_components/zhuaclfrontend/assets/dist/main.js",
		"./bower_components/zhulog/dist/main.js",
		"./bower_components/tek-libraries/dist/main.js"
	];

    grunt.initConfig({
		concat: {
			libs:{
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: libs,
				dest: './assets/js/libs.js'
			},
			js: {
				options: {
					process: function (src, filepath) {
						return '// FILE: ' + filepath + '\n' + src + '\n';
					}
				},
				src: getJSPaths(),
				dest: './assets/js/app.js'
			},
			css: {
				options: {
					process: function (src, filepath) {
						return '/* FILE: ' + filepath + ' */\n' + src + '\n';
					}
				},
				src: getCSSPaths(),
				dest: './assets/css/app.css'
			}
		},
		uglify: {
			options: {
				mangle: false,
				compress: false,
				report: 'min',
				// the banner is inserted at the top of the output
				banner: '/*! <%= grunt.template.today("dd-mm-yyyy") %> */\n'
			},
			dist: {
				files: {
					'./assets/js/app.min.js': getJSPaths()
				}
			}
		},
		jshint: {
			all: getJSPaths().concat(["GruntFile.js"]),
			options: {
				evil: true,
				esversion: 6
			}
		},
		watch: {
			js: {
				files: getJSPaths().concat(["GruntFile.js"]),
				tasks: ['jshint', 'concat:js']
			},
			css: {
				files: getCSSPaths(),
				tasks: ['concat:css']
			}
		},
		cssmin: {
			dist: {
				src: getCSSPaths(),
				dest: './assets/css/app.min.css'
			}
		},
		// NÃO COLOCAR PROPRIEDADE PARA SUBSTIUIR IDS
		zhIdGenerator: {
			options: {
				jsonPath: getJSONPaths(),
				menuPath: ["./assets/json/menu.json"]
			}
		},
		buildAclStruct : {
            options: {
                menuConfigFile: "./menuConfig.json"
            }
        },
		buildEnvironment: {
			development: {
				configFile: "./config/devEnvironment.json"
			},
			production: {
				configFile: "./config/prodEnvironment.json"
			},
			test: {
				configFile: "./config/testEnvironment.json",
				options: {
					force: true
				}
			},
			srcFolders: ["../backend/config/", "./assets/js", "./assets/json", "./config", "./", "./assets", "../../teknisa/cmp/reports"]
		},
		zhWords: {
			options: {
				serverUrl: "https://translate.teknisa.com/backend/index.php",
				organization: "20",
				project: "2231"
			}
		}
	});

	function getJSPaths() {
		var paths = [];
		directories.forEach(function (directory) {
		    paths.push('./assets/templates/js/**/*.js');
			paths.push('./assets/js/' + directory + '/**/*.js');
			subModules.forEach(function (subModule) {
			    paths.push('./assets/modules/' + subModule + '/assets/templates/js/**/*.js');
				paths.push('./assets/modules/' + subModule + '/assets/js/' + directory + '/**/*.js');
			});
		});
		return paths;
	}

	function getCSSPaths() {
		var paths = [];
		paths.push('./assets/templates/css/**/*.css');
		paths.push('./assets/css/**/*.css');
		paths.push('!./assets/css/app.css');
		
		subModules.forEach(function (subModule) {
		    paths.push('./assets/modules/' + subModule + '/assets/templates/css/**/*.css');
			paths.push('./assets/modules/' + subModule + '/assets/css/**/*.css');
			paths.push('!./assets/modules/' + subModule + '/assets/css/app.css');
		});
		return paths;
	}

	function getJSONPaths() {
		var paths = ["./assets/json"];
		subModules.forEach(function (subModule) {
			paths.push("./assets/modules/" + subModule + "/assets/json");
		});
		return paths;
	}

	function buildFileContent(srcFiles, filePath, directory) {
		var parts = filePath.split('/');
		parts.shift();
		parts.shift();
		var filename = parts[parts.length - 1];
		var extension = filename.split('.').pop().toUpperCase();
		if (extension == 'JSON' || extension == 'XML') {
			if (directory == 'datasources') {
				filename = parts.pop();
				filename = filename.replace(/\.[^/.]+$/, "");
				srcFiles[directory][filename] = './../../../' + parts.join('/');
			} else {
				srcFiles[directory].push('/../../' + parts.join('/'));
			}
		}
	}

	function buildSubModules(subModules, directory, srcFiles) {
		subModules.forEach(function (subModule) {
			var subModulesChild = grunt.file.expand({
				filter: 'isDirectory'
			}, subModule + '/Modules/*');

			if (subModulesChild.length)
				buildSubModules(subModulesChild, directory, srcFiles);

			grunt.file.expand({
					filter: 'isFile'
				}, subModule + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
		});
	}

	grunt.registerTask('backendStep', 'build backend config files', function () {
		var srcFiles = {
			config: [],
			datasources: {}
		};
		var directories = ['config', 'datasources'];
		var dest = {
			config: '../backend/config/servicesMapping.json',
			datasources: '../backend/config/dataSourcesMapping.json'
		};
		directories.forEach(function (directory) {
			backendModules.forEach(function (module) {
				var moduleParts = module.split('/');
				var moduleName = moduleParts[moduleParts.length - 1].toUpperCase();
				var subModules = grunt.file.expand({
					filter: 'isDirectory'
				}, module + '/src/Teknisa/' + moduleName + '/Modules/*');

				buildSubModules(subModules, directory, srcFiles);

				grunt.file.expand({
					filter: 'isFile'
				}, module + '/' + directory + '/**/*').forEach(function (filePath) {
					buildFileContent(srcFiles, filePath, directory);
				});
			});
		});
		grunt.file.write(dest.config, JSON.stringify(srcFiles.config, null, 2));
		grunt.file.write(dest.datasources, JSON.stringify(srcFiles.datasources, null, 2));
	});

	grunt.loadNpmTasks('grunt-contrib-jshint');
	grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks('grunt-contrib-concat');
	grunt.loadNpmTasks('grunt-contrib-watch');
	grunt.loadNpmTasks('grunt-contrib-cssmin');
	grunt.loadNpmTasks('grunt-zh-id-generator');
	grunt.loadNpmTasks('grunt-zhuacl-structure');
	grunt.loadNpmTasks('grunt-zh-environment-builder');
	grunt.loadNpmTasks('grunt-zh-words');

	grunt.registerTask('default', ['zhWords', 'jshint', 'backendStep', 'concat', 'buildAclStruct', 'buildEnvironment']);
	grunt.registerTask('dev', ['backendStep', 'concat', 'buildEnvironment']);
	grunt.registerTask('mergeBranch', ['zhIdGenerator', 'buildAclStruct']);
	grunt.registerTask('final', ['cssmin', 'uglify']); 
};