function Utilities(ScreenService, ZhPreferences, Query, ZhFilterPreferences, validateEngine, templateManager, maskEngine, ZhUtil) {

    this.changeTagRichText = function(fieldValue) {
        if (typeof(fieldValue) === 'string') {
            var newFieldValue;

            if (fieldValue.substr(0, 3) == '<p>') {
                newFieldValue = fieldValue.replace(/<p>/g, '').replace(/<\/p>/g, '<br>');
            }
            else if (fieldValue.substr(0, 3) == '<p ') {
                newFieldValue = fieldValue.replace(/<p/g, '<span').replace(/<\/p>/g, '</span><br>');
            }
            else {
                newFieldValue = fieldValue;
            }

            if (fieldValue.substr(-4) == '</p>') {
                newFieldValue = newFieldValue.slice('', -4);
            }

            return newFieldValue;
        }
        else {
            return fieldValue; 
        }
    };
    
    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };
                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            }
            else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };

    function validaProduto(value, validationAttrs, field) {

        var filtro = templateManager.container.getActiveWidget();
        var retorno = {
            "valid": true
        };
        field = validationAttrs.field;
        let fieldProduto;
        const final = filtro.currentRow.CDPRODFIN;
        const inicial = filtro.currentRow.CDPRODINI;

        if (value) {
            fieldProduto = field == "NMPRODFIN" ? inicial : final;
            if (fieldProduto) {
                if (inicial > final) {
                    return {
                        "valid": false,
                        "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n('Produto inicial deve ser menor ou igual ao produto final')
                    };
                }
            }
        }
        else if (field == "NMPRODFIN" && filtro.currentRow.CDPRODINI || field == "NMPRODINI" && filtro.currentRow.CDPRODFIN) {
            let msg = field == "NMPRODFIN" ? 'Produto final obrigatório' : 'Produto inicial obrigatório';
            return {
                "valid": false,
                "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n(msg)
            };
        }

        return retorno;
    }

    var registeredValidations = validateEngine.getMappedValidations();

    if (!registeredValidations.hasOwnProperty('validaProduto')) {
        validateEngine.registerValidation('validaProduto', validaProduto);
    }
    
    /* Cria máscara para o campo 'CDNMPRODUTO', onde é recebido o nome e o código
     * do produto concatenados.
     */
    var maskCodigoNomeProduto = function(value) {
        if (value) {
            var mascara = getLocalVar('CDPICTPROD');
            var tamanhoCodigo = mascara.replace(/\./g, '').length;
            var cdProduto = value.substring(0, tamanhoCodigo);
            var mask = {
                type: "fix",
                params: {
                    mask: mascara
                }
            };
            var cdProdutoFormatado = Util.applyMask(cdProduto, mask, maskEngine, ZhUtil);
            return value.replace(cdProduto, cdProdutoFormatado);
        }
    };

    maskEngine.registerMask('CDNMPROD',
        function() {},
        maskCodigoNomeProduto
    );
    
    let stateDoubleClick = {
        clicks: 0,
        lastRowIndex: 0,
        currentRowIndex: 0,
    };
    
    let intervalId;
    
    /**
     * Permite utilizar duplo clique no grid para acionar uma determinada função.
     * 
     * @param {function} execFunction - Função a ser executada no clique duplo.
     * @param {array} params - Array contendo os parâmetros a serem passados.
     */
    this.onDoubleClick = function(execFunction, params) {
        const actualWidget = templateManager.container.getActiveWidget();
        
        stateDoubleClick.currentRowIndex = actualWidget.currentRow.__index;
        
        if (stateDoubleClick.clicks == 0) {
            stateDoubleClick.lastRowIndex = actualWidget.currentRow.__index;
            
            intervalId = setInterval(() => {
                clearInterval(intervalId);
                
                if (stateDoubleClick.clicks > 1 && stateDoubleClick.lastRowIndex == stateDoubleClick.currentRowIndex) {
                    execFunction(...params);
                }
                
                stateDoubleClick.clicks = 0;
            }, 500);
        }
        
        stateDoubleClick.clicks++;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});