function Utilities(ScreenService, General, ZhPreferences, Query, validateEngine, ZhFilterPreferences, templateManager) {

/*global Util, Configuration, $*/

    this.changeTagRichText = function(fieldValue) {
        if (typeof(fieldValue) === 'string') {
            var newFieldValue;

            if (fieldValue.substr(0, 3) == '<p>') {
                newFieldValue = fieldValue.replace(/<p>/g, '').replace(/<\/p>/g, '<br>');
            }
            else if (fieldValue.substr(0, 3) == '<p ') {
                newFieldValue = fieldValue.replace(/<p/g, '<span').replace(/<\/p>/g, '</span><br>');
            }
            else {
                newFieldValue = fieldValue;
            }

            if (fieldValue.substr(-4) == '</p>') {
                newFieldValue = newFieldValue.slice('', -4);
            }

            return newFieldValue;
        }
        else {
            return fieldValue;
        }
    };

    this.startFilter = function(widget, currentRow) {
        return ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };

                    currentRow = filter.FILTER_ROW;

                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if (defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            }
            else {
                if (Util.isUndefinedOrNull(currentRow) || Util.isEmptyObject(currentRow)) {
                    currentRow = buildFilterCurrentRow(widget);
                }
            }
            ZhFilterPreferences.openFilterWidget(widget, currentRow);
        });
    };

    var buildFilterCurrentRow = function(widget) {
        return getFilterRowWithDefaultValues(widget.fields);
    };

    var getFilterRowWithDefaultValues = function(filterFields) {
        let filterRow = {};
        filterFields.forEach(field => {
            let defaultValue = Util.getNestedProperty(field, ["filterProperties", "defaultValue"]) || field.defaultValue;
            if (!Util.isUndefinedOrNull(defaultValue)) {
                filterRow[field.name] = Util.evaluate(defaultValue);
            }
        });
        return filterRow;
    };
    
    function formatStringToFloat(word, simbolThousands, simbolDecimals){
        word = word.split(simbolThousands).join('');
        word = word.replace(simbolDecimals, '.');    
        return word;
    }


    function verifyRange(value, validationAttrs) {
        const maxValue = validationAttrs.max || '9999999.99';
        const minValue = validationAttrs.min || 0;
        const simbolThousands = validationAttrs.thousands || '.';
        const simbolDecimals = validationAttrs.decimals || ',';
        let valid = true;
        let msg = ScreenService.i18n(validationAttrs.message) || ScreenService.i18n("Não está dentro do intervalo de " + minValue + " a " + maxValue);
        
        if (typeof(value) == 'string') {
            value = formatStringToFloat(value, simbolThousands, simbolDecimals);
        }

        if (value && (parseFloat(value) > parseFloat(maxValue) || parseFloat(value) < parseFloat(minValue))) {
            valid = false;
        }
        
        return  {"valid": valid, "message": msg};
    }
    
    function validaProduto(value, validationAttrs) {

        var filtro = templateManager.container.getActiveWidget();
        var retorno = {
            "valid": true
        };
        let field = validationAttrs.field;
        let fieldProduto;
        const final = filtro.currentRow.CDPRODFIN;
        const inicial = filtro.currentRow.CDPRODINI;

        if (value) {
            fieldProduto = field == "NMPRODFIN" ? inicial : final;
            if (fieldProduto) {
                if (inicial > final) {
                    return {
                        "valid": false,
                        "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n('Produto inicial deve ser menor ou igual ao produto final')
                    };
                }
            }
        }
        else if (field == "NMPRODFIN" && filtro.currentRow.CDPRODINI || field == "NMPRODINI" && filtro.currentRow.CDPRODFIN) {
            let msg = field == "NMPRODFIN" ? 'Produto final obrigatório' : 'Produto inicial obrigatório';
            return {
                "valid": false,
                "message": ScreenService.i18n(validationAttrs.message) || ScreenService.i18n(msg)
            };
        }

        return retorno;
    }
    
    var registeredValidations = validateEngine.getMappedValidations();
    
    if (!registeredValidations.hasOwnProperty('verifyRange')) {
        validateEngine.registerValidation('verifyRange', verifyRange);
    }
    
    
    if (!registeredValidations.hasOwnProperty('validaProduto')) {
        validateEngine.registerValidation('validaProduto', validaProduto);
    }
}

Configuration(function(ContextRegister) {
    ContextRegister.register('Utilities', Utilities);
});